package io.provis;

import java.lang.annotation.Annotation;
import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import javax.inject.Inject;
import javax.inject.Named;

import org.sonatype.guice.bean.locators.BeanLocator;

import com.google.inject.Key;
import com.google.inject.name.Names;

@Named
public class Lookup {

  private BeanLocator locator;

  @Inject
  public Lookup(BeanLocator locator) {
    this.locator = locator;
  }
  
  // ----------------------------------------------------------------------
  // Container lookup methods
  // ----------------------------------------------------------------------

  public final <T> T lookup(final Class<T> type) {
    return lookup(Key.get(type));
  }

  public final <T> T lookup(final Class<T> type, final String name) {
    return lookup(type, Names.named(name));
  }

  public final <T> T lookup(final Class<T> type, final Class<? extends Annotation> qualifier) {
    return lookup(Key.get(type, qualifier));
  }

  public final <T> T lookup(final Class<T> type, final Annotation qualifier) {
    return lookup(Key.get(type, qualifier));
  }

  // ----------------------------------------------------------------------
  // Implementation methods
  // ----------------------------------------------------------------------

  private final <T> T lookup(final Key<T> key) {
    final Iterator<? extends Entry<Annotation, T>> i = locator.locate(key).iterator();
    return i.hasNext() ? i.next().getValue() : null;
  }

  public static void setObjectProperty(Object o, String propertyName, Object value) {
    Class<?> c = o.getClass();

    // First see if name is a property ala javabeans
    String methodSuffix = Character.toUpperCase(propertyName.charAt(0)) + propertyName.substring(1, propertyName.length());
    String methodName = "set" + methodSuffix;
    Class<? extends Object> type = value.getClass();
    
    if (List.class.isAssignableFrom(type)) {
      type = List.class;
    } else if (Map.class.isAssignableFrom(type)) {
      type = Map.class;
    } else if(Boolean.class.isAssignableFrom(type)) {
      // We need to use an Object but we need to look at the target field
      type = boolean.class;
    }

    Method m = getMethod(c, methodName, new Class[] {
      type
    });
    if (m != null) {
      try {
        invokeMethod(m, o, value);
      } catch (Exception e) {
        System.out.println("Can't set property " + propertyName + " using method set" + methodSuffix + " from " + c.getName() + " instance: " + e);
      }
    } else {
      // try for a visible field
      try {
        Field f = c.getField(propertyName);
        try {
          f.set(o, value);
        } catch (IllegalAccessException iae) {
          System.out.println("Can't access property " + propertyName + " using direct field access from " + c.getName() + " instance: " + iae);
        }
      } catch (NoSuchFieldException nsfe) {
        System.out.println("Class " + c.getName() + " has no such property/field: '" + propertyName + "': " + nsfe);
      }
    }
  }

  protected static Object newInstance(String name) {
    Object o = null;
    try {
      o = Class.forName(name).newInstance();
    } catch (Exception e) {
      System.out.println("can't make instance of " + name);
    }
    return o;
  }

  protected static Method getMethod(Class c, String methodName, Class[] args) {
    Method m;
    try {
      m = c.getMethod(methodName, args);
    } catch (NoSuchMethodException nsme) {
      m = null;
    }
    return m;
  }

  protected static Object invokeMethod(Method m, Object o) throws IllegalAccessException, InvocationTargetException {
    return invokeMethod(m, o, null);
  }

  protected static Object invokeMethod(Method m, Object o, Object value) throws IllegalAccessException, InvocationTargetException {
    Object[] args = null;
    if (value != null) {
      args = new Object[] {
        value
      };
    }
    value = m.invoke(o, args);
    return value;
  }
}
