package io.provis.model;

import io.provis.model.Action;
import io.provis.model.ProvisoArtifact;

import java.io.File;
import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

public class ArtifactSetOrg {

  private ArtifactSetOrg parent;
  //
  // The name of the directory
  //
  private String outputDirectory;
  //
  // The real directory based on the parent's directory
  //
  private File actualOutputDirectory;
  //
  // Child filesets
  //
  private Map<String, ArtifactSetOrg> children;
  private List<String> excludes;
  private Map<String, ProvisoArtifact> artifactsMapKeyedByGA;
  private Map<String, ProvisoArtifact> resolvedArtifacts;
  private List<Action> actions;

  public ArtifactSetOrg() {
    this(null);
  }

  public ArtifactSetOrg(String outputDirectory) {
    this(outputDirectory, null);
  }

  public ArtifactSetOrg(String outputDirectory, ArtifactSetOrg parent) {
    this.outputDirectory = outputDirectory;
    this.parent = parent;
    excludes = new ArrayList<String>();
    children = new LinkedHashMap<String, ArtifactSetOrg>();
    artifactsMapKeyedByGA = new LinkedHashMap<String, ProvisoArtifact>();
    actions = new ArrayList<Action>();
  }

  //
  // Parent
  //
  public ArtifactSetOrg getParent() {
    return parent;
  }

  //
  // Coordinates
  //
  public Map<String, ProvisoArtifact> getArtifactMapKeyedByGA() {
    return artifactsMapKeyedByGA;
  }

  public List<ProvisoArtifact> getArtifacts() {
    return new ArrayList<ProvisoArtifact>(artifactsMapKeyedByGA.values());
  }

  public void addArtifact(ProvisoArtifact coordinate) {
    artifactsMapKeyedByGA.put(coordinate.getGA(), coordinate);
  }

  public void setArtifacts(List<String> artifacts) {
    for (String coordinate : artifacts) {
      addArtifact(coordinate);
    }
  }

  public void addArtifact(String coordinate) {
    addArtifact(new ProvisoArtifact(coordinate));
  }

  //
  // outputDirectory
  //
  public String getDirectory() {
    return outputDirectory;
  }

  public void setDirectory(String outputDirectory) {
    this.outputDirectory = outputDirectory;
  }

  //
  // Children
  //
  public List<ArtifactSetOrg> getFileSets() {
    return new ArrayList<ArtifactSetOrg>(children.values());
  }

  public void setFileSets(List<ArtifactSetOrg> fileSets) {
    for (ArtifactSetOrg child : fileSets) {
      addFileSet(child);
    }
  }

  public void addFileSet(ArtifactSetOrg child) {
    //
    // Set the parent
    //
    child.setParent(this);
    //
    // Do some inheritance.
    //
    for (String exclude : getExcludes()) {
      //
      // Add the exclusion unless it is an artifact explicity added by the child
      //
      if (!child.getArtifactMapKeyedByGA().containsKey(exclude)) {
        child.addExclude(exclude);
      }
    }

    children.put(child.getDirectory(), child);
  }

  public void setParent(ArtifactSetOrg parent) {
    this.parent = parent;
  }

  public List<String> getExcludes() {
    return excludes;
  }

  public void setExcludes(List<String> excludes) {
    this.excludes = excludes;
  }

  public void addExclude(String exclude) {
    excludes.add(exclude);
  }

  public Map<String, ProvisoArtifact> getResolvedArtifacts() {
    return resolvedArtifacts;
  }

  public void setResolvedArtifacts(Map<String, ProvisoArtifact> resolvedArtifacts) {
    this.resolvedArtifacts = resolvedArtifacts;
  }

  public String toString() {
    return outputDirectory;
  }

  public void addAction(Action action) {
    actions.add(action);
  }

  public List<Action> getActions() {
    return actions;
  }

  public ArtifactSetOrg getFileSet(String string) {
    return children.get(string);
  }

  public File getActualOutputDirectory() {
    return actualOutputDirectory;
  }

  public void setActualOutputDirectory(File actualOutputDirectory) {
    this.actualOutputDirectory = actualOutputDirectory;
  }
}
