package io.provis.model;

import java.util.ArrayList;
import java.util.List;

import org.sonatype.aether.artifact.Artifact;
import org.sonatype.aether.util.artifact.DefaultArtifact;

public class ProvisoArtifact extends DelegatingArtifact {

  private List<Action> actions;

  public ProvisoArtifact(String coordinate) {
    this(getArtifact(coordinate));
  }

  public ProvisoArtifact(Artifact artifact) {
    super(artifact);
    actions = new ArrayList<Action>();
  }

  private static Artifact getArtifact(String coordinate) {
    return new DefaultArtifact(resolveVersion(coordinate));
  }

  //TODO: this is funky and there should be no magic here
  private static String resolveVersion(String artifact) {
    if (nthOccurrence(artifact, ':', 1) == -1) {
      return artifact + ":" + "NOTSET";
    }
    return artifact;
  }

  //
  // zero-based nth occurrence
  //
  public static int nthOccurrence(String str, char c, int n) {
    int pos = str.indexOf(c, 0);
    while (n-- > 0 && pos != -1) {
      pos = str.indexOf(c, pos + 1);
    }
    return pos;
  }

  public List<Action> getActions() {
    return actions;
  }

  public ProvisoArtifact addAction(Action action) {
    actions.add(action);
    return this;
  }

  public void setActions(List<Action> actions) {
    this.actions = actions;
  }

  @Override
  protected DelegatingArtifact newInstance(Artifact delegate) {
    ProvisoArtifact artifact = new ProvisoArtifact(delegate);
    artifact.setActions(actions);
    return artifact;
  }

  //
  // The only thing using this is the lifecycle participant in Maven and it is expecting G:A, so this is a bit of a misnomer
  //
  public String getCoordinate() {

    StringBuffer sb = new StringBuffer();
    sb.append(getGroupId()).append(":").append(getArtifactId()).append(":");

    if (!getExtension().isEmpty()) {
      sb.append(getExtension()).append(":");
    }

    if (!getClassifier().isEmpty()) {
      sb.append(getClassifier()).append(":");
    }

    sb.append(getVersion());

    return sb.toString();
  }

  public String getGA() {
    return getGroupId() + ":" + getArtifactId();
  }

  public String getGAV() {
    return getGroupId() + ":" + getArtifactId() + ":" + getVersion();
  }

}
